# SPDX-FileCopyrightText: 2024-present Matthias Kestenholz <mk@feinheit.ch>
#
# SPDX-License-Identifier: MIT

try:
    from blacknoise._impl import BlackNoise  # noqa
except ModuleNotFoundError:
    pass
